﻿<?xml version="1.0" encoding="utf-8" standalone="yes"?>
<!-- Mapcam.info Garmin GPX output fixer & filter by warnings type -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:gpx="http://www.topografix.com/GPX/1/1" xmlns:gpxx="http://www.garmin.com/xmlschemas/GpxExtensions/v3"
                exclude-result-prefixes="gpx">
    <xsl:output method="xml" encoding="utf-8" indent="yes" standalone="yes"/>

    <!-- Warning type Id to filter source speedcam.gpx; should be overridden externally -->
    <xsl:param name="WarningTypeId">5</xsl:param>
    
    <!-- Warning tag name; could be overridden externally; could be empty to exclude tags insertion -->
    <xsl:param name="WarningTagName">
        <xsl:choose>
            <xsl:when test="$WarningTypeId=1">Radar</xsl:when>
            <xsl:when test="$WarningTypeId=2">Radar-traffic light</xsl:when>
            <xsl:when test="$WarningTypeId=3">Radar (red light)</xsl:when>
            <xsl:when test="$WarningTypeId=4">Radar (pair)</xsl:when>
            <xsl:when test="$WarningTypeId=5">Mobile radar</xsl:when>
            <xsl:when test="$WarningTypeId=20">Stationary post</xsl:when>
            <xsl:when test="$WarningTypeId=21">Railroad crossing</xsl:when>
            <xsl:when test="$WarningTypeId=22">School</xsl:when>
            <xsl:when test="$WarningTypeId=100">Built-up area</xsl:when>
            <xsl:when test="$WarningTypeId=101">Speed control</xsl:when>
            <xsl:when test="$WarningTypeId=102">Speed bump</xsl:when>
            <xsl:when test="$WarningTypeId=103">Bad road</xsl:when>
            <xsl:when test="$WarningTypeId=104">Dangerous turn</xsl:when>
            <xsl:when test="$WarningTypeId=105">Dangerous crossroad</xsl:when>
            <xsl:when test="$WarningTypeId=106">Other danger</xsl:when>
        </xsl:choose>
    </xsl:param>

    <!-- Add speed suffix to warning audio file name; have 0 or 1 values; could be overridden externally -->
    <xsl:param name="AddSpeedToWarningAudioFileName">
        <xsl:choose>
            <xsl:when test="$WarningTypeId=1">1</xsl:when>
            <xsl:when test="$WarningTypeId=101">1</xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
        </xsl:choose>
    </xsl:param>

    <!-- Add type suffix (1-8) to warning audio file name; have 0 or 1 values; could be overridden externally; ignored if AddSpeedToWarningAudioFileName=1 -->
    <xsl:param name="AddTypeToWarningAudioFileName">
        <xsl:choose>
            <xsl:when test="$WarningTypeId=5">1</xsl:when>
            <xsl:otherwise>0</xsl:otherwise>
        </xsl:choose>
    </xsl:param>

    <xsl:template match="/">
        <xsl:apply-templates select="gpx:gpx"/>
    </xsl:template>

    <xsl:template match="gpx:gpx">
        <xsl:copy>
            <xsl:copy-of select="@creator"/>
            <xsl:copy-of select="@version"/>
            <xsl:attribute name="schemaLocation" namespace="http://www.w3.org/2001/XMLSchema-instance">http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd http://www.garmin.com/xmlschemas/GpxExtensions/v3 http://www.garmin.com/xmlschemas/GpxExtensions/v3/GpxExtensionsv3.xsd</xsl:attribute>
            <xsl:copy-of select="gpx:metadata"/>
            <xsl:apply-templates select="gpx:wpt[gpx:link/@href=concat($WarningTypeId, '.bmp') or starts-with(gpx:link/@href, concat($WarningTypeId, '_'))]"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="gpx:wpt">
        <xsl:copy>
            <xsl:copy-of select="@lat"/>
            <xsl:copy-of select="@lon"/>
            <xsl:apply-templates select="gpx:name"/>
            <xsl:apply-templates select="gpx:cmt"/>
            <xsl:apply-templates select="gpx:desc"/>
            <!-- Process links to .bmp icons only; links to .wav files will be completely regenerated -->
            <xsl:apply-templates select="gpx:link[substring-after(@href, '.')='bmp']"/>
            <xsl:copy-of select="gpx:sym"/>
            <xsl:copy-of select="gpx:extensions"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="gpx:name">
        <!-- Cut underscore from name and fix Russian 'Speed bump' name -->
        <xsl:variable name="name" select="translate(., '_', ' ')"/>
        <xsl:copy>
            <xsl:choose>
                <xsl:when test="contains($name, 'полицейски@')">
                    <xsl:value-of select="substring-before($name,'полицейски@')"/>
                    <xsl:text>полицейский@</xsl:text>
                    <xsl:value-of select="substring-after($name,'полицейски@')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$name"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy> 
    </xsl:template>

    <!-- TODO Exchange output <cmt> and <desc> nodes filling? -->
    
    <xsl:template match="gpx:cmt">
        <!-- Cut inserted tag in brackets (if any) from user comment string -->
        <xsl:variable name="desc">
            <xsl:choose>
                <xsl:when test="contains(., ']')">
                    <xsl:value-of select="normalize-space(substring-after(., ']'))"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="normalize-space(.)"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:if test="string-length($desc) &gt; 0">
            <xsl:copy>
                <xsl:value-of select="$desc"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>

    <xsl:template match="gpx:desc">
        <!-- Insert appropriate tag and cut user comments -->
        <xsl:if test="string-length($WarningTagName) &gt; 0">
            <xsl:variable name="speed" select="substring-after(../gpx:name, '@')"/>
            <xsl:copy>
                <xsl:text>[</xsl:text>
                <xsl:value-of select="$WarningTagName"/>
                <xsl:text> </xsl:text>
                <xsl:value-of select="$speed"/>
                <xsl:text> km/h]</xsl:text> <!-- TODO Use 'kph' units name? -->
            </xsl:copy>
        </xsl:if>
    </xsl:template>

    <xsl:template match="gpx:link">
        <!-- Create two 'link' elements with references to .bmp and .wav files -->
        <xsl:copy>
            <xsl:copy-of select="@href"/>
        </xsl:copy>
        <xsl:copy>
            <xsl:attribute name="href">
                <xsl:value-of select="$WarningTypeId"/>
                <xsl:choose>
                    <xsl:when test="$AddSpeedToWarningAudioFileName=1">
                        <xsl:variable name="speed" select="substring-after(../gpx:name, '@')"/>
                        <xsl:text>_</xsl:text>
                        <xsl:value-of select="$speed"/>
                    </xsl:when>
                    <xsl:when test="$AddTypeToWarningAudioFileName=1">
                        <xsl:variable name="desc" select="../gpx:cmt"/>
                        <xsl:choose>
                            <xsl:when test="contains($desc, 'кирпич') or contains($desc, 'запрещено')">
                                <xsl:text>_1</xsl:text>
                            </xsl:when>
                            <xsl:when test="contains($desc, 'встреч') or contains($desc, 'Встреч') or contains($desc, 'двойн')">
                                <xsl:text>_2</xsl:text>
                            </xsl:when>
                            <xsl:when test="contains($desc, 'стоп') or contains($desc, 'STOP') or contains($desc, 'красный')">
                                <xsl:text>_3</xsl:text>
                            </xsl:when>
                            <xsl:when test="contains($desc, 'обгон') or contains($desc, 'Обгон')">
                                <xsl:text>_4</xsl:text>
                            </xsl:when>
                            <xsl:when test="contains($desc, 'пешеход') or contains($desc, 'переход')">
                                <xsl:text>_5</xsl:text>
                            </xsl:when>
                            <xsl:when test="contains($desc, 'скорост') or contains($desc, 'Скорост') or contains($desc, 'превышение') or contains($desc, 'ограничен') or contains($desc, 'тренога') or contains($desc, 'Тренога') or contains($desc, 'радар') or contains($desc, 'КРИС') or contains($desc, ' фен') or contains($desc, 'гонщик')">
                                <xsl:text>_6</xsl:text>
                            </xsl:when>
                            <xsl:when test="contains($desc, ' ряд') or contains($desc, 'сплошн') or contains($desc, 'разделительн')">
                                <xsl:text>_7</xsl:text>
                            </xsl:when>
                            <xsl:when test="contains($desc, 'стрелк')">
                                <xsl:text>_8</xsl:text>
                            </xsl:when>
                        </xsl:choose>
                    </xsl:when>
                </xsl:choose>
                <xsl:text>.wav</xsl:text>
            </xsl:attribute>
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
